import time
from threading import Thread


class ProcessTime:
    def __init__(self):
        self.tic = self()
        self.last_run = 0
        self.first_run = True

    def running_time(self, print_: bool = False, reset_mode=False):
        toc = self() - self.tic
        process_time = int(toc * 1000)

        if print_:
            print("Process Time: ", process_time, " ms")

        if reset_mode:
            self.tic = self()

        return process_time

    def timeout(self, cycle_ms: int):
        if self.last_run == 0:
            self.last_run = self()

        time_difference = self() - self.last_run

        if time_difference >= cycle_ms * 0.001:
            self.last_run = self()

            return True
        return False

    def reset(self):
        self.last_run = 0
        self.first_run = True

    def __call__(self):
        return time.perf_counter()


class Loop:
    def __init__(self, timeout: int = 100):
        self.timeout = timeout

    @staticmethod
    def pause(milliseconds: int):
        time.sleep(milliseconds * 0.001)

    def __call__(self):
        self.pause(self.timeout)
        return True


class ThreadIt(Thread):
    def __init__(self, group=None, target=None, name=None,
                 args=(), kwargs=None, verbose=None):
        Thread.__init__(self, group, target, name, args, kwargs)
        if kwargs is None:
            kwargs = {}

        self._target = target
        self._args = args
        self._kwargs = kwargs
        self._return = []

        self.start()

    def run(self):
        if self._target is not None:
            self._return = self._target(*self._args, **self._kwargs)

    def join(self, *args):
        Thread.join(self, *args)
        return self._return


if __name__ == "__main__":
    t = ProcessTime()
    while True:
        if t.timeout(5000):
            print(t.running_time())



import ctypes
import math
from random import randint
from time import sleep

MOUSEEVENTF_MOVE = 0x0001  # mouse move
MOUSEEVENTF_LEFTDOWN = 0x0002  # left button down
MOUSEEVENTF_LEFTUP = 0x0004  # left button up
MOUSEEVENTF_RIGHTDOWN = 0x0008  # right button down
MOUSEEVENTF_RIGHTUP = 0x0010  # right button up
MOUSEEVENTF_MIDDLEDOWN = 0x0020  # middle button down
MOUSEEVENTF_MIDDLEUP = 0x0040  # middle button up
MOUSEEVENTF_WHEEL = 0x0800  # wheel button rolled
MOUSEEVENTF_ABSOLUTE = 0x8000  # absolute move
SM_CXSCREEN = 0
SM_CYSCREEN = 1

ctypes.windll.user32.SetProcessDPIAware()
screen_width = ctypes.windll.user32.GetSystemMetrics(SM_CXSCREEN)
screen_height = ctypes.windll.user32.GetSystemMetrics(SM_CYSCREEN)
__event = ctypes.windll.user32.mouse_event


def click(right: bool = False, left: bool = False, middle: bool = False, double_click: bool = False):
    if double_click:
        down(right=right, left=left, middle=middle)
        delay()
        up(right=right, left=left, middle=middle)

        delay(150)

        down(right=right, left=left, middle=middle)
        delay()
        up(right=right, left=left, middle=middle)

    else:
        down(right=right, left=left, middle=middle)
        delay()
        up(right=right, left=left, middle=middle)


def down(right: bool = False, left: bool = False, middle: bool = False):
    left = not any((right, left, middle))
    press = __get_button(right=right, left=left, middle=middle)
    __event(press[0], 0, 0, 0, 0)


def up(right: bool = False, left: bool = False, middle: bool = False):
    left = not any((right, left, middle))
    press = __get_button(right=right, left=left, middle=middle)
    __event(press[1], 0, 0, 0, 0)


def move(x: int, y: int):
    x, y = __normalize_xy(x, y)
    __event(MOUSEEVENTF_MOVE + MOUSEEVENTF_ABSOLUTE, x, y, 0, 0)


def __normalize_xy(x: int, y: int):
    old_position = get_position()
    x = x if x != -1 else old_position[0]
    y = y if y != -1 else old_position[1]
    x = math.floor(65536 * x / screen_width + 1)
    y = math.floor(65536 * y / screen_height + 1)

    return x, y


def __get_button(right: bool = False, left: bool = False, middle: bool = False):
    if sum((right, left, middle)) > 1:
        raise Error("Error: only 1 button option is allowed!")
    if right:
        return MOUSEEVENTF_RIGHTDOWN, MOUSEEVENTF_RIGHTUP
    if left:
        return MOUSEEVENTF_LEFTDOWN, MOUSEEVENTF_LEFTUP
    if middle:
        return MOUSEEVENTF_MIDDLEDOWN, MOUSEEVENTF_MIDDLEUP


def is_pressed(code):
    return ctypes.windll.user32.GetKeyState(code) & 0x8000


def get_position():
    p = Point()
    ctypes.windll.user32.GetCursorPos(ctypes.byref(p))

    return p.x, p.y


def delay(seconds: float = 0):
    if seconds == 0:
        sleep((randint(50, 60) * 0.001))
    else:
        sleep(seconds)


class Point(ctypes.Structure):
    _fields_ = [("x", ctypes.c_long), ("y", ctypes.c_long)]


class Error(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message



import cv2
import numpy as np

import ninja_tools.utils as u


def find_image(haystack: np.ndarray,
               needle: np.ndarray, threshold: float = 8,
               passed_only: bool = True, get_rect=False,
               get_center: bool = False, get_dist=False,
               show: bool = False):
    result = cv2.matchTemplate(haystack, needle, cv2.TM_CCOEFF_NORMED)
    passed = np.amax(result) >= (threshold * 0.1)

    w_src, h_src = haystack.shape[1], haystack.shape[0]
    w, h = needle.shape[1], needle.shape[0]

    if get_center or get_rect or show:
        locations = np.where(result >= (threshold * 0.1))
        locations = list(zip(*locations[::-1]))

        if not locations:
            return False

        rectangles = []

        for loc in locations:
            rect = [int(loc[0]), int(loc[1]), w, h]
            # Add every box to the list twice in order to retain single (non-overlapping) boxes
            rectangles.append(rect)
            rectangles.append(rect)

        rectangles, _ = cv2.groupRectangles(rectangles, groupThreshold=1, eps=0.5)

        points = []

        if show:
            for pt in locations:  # Switch columns and rows
                cv2.rectangle(haystack, pt, (pt[0] + w, pt[1] + h), (0, 0, 255), 2)

            print(f"Max match: {np.amax(result):,.2%}")

            cv2.imshow(None, haystack)
            if cv2.waitKey(1) & 0xFF == ord('q'):
                cv2.destroyAllWindows()

        # Loop over all the rectangles
        for (x, y, w, h) in rectangles:
            if get_rect:
                points.append((x, y, x + w, y + h))

            else:
                # Determine the center position
                center_needle = x + int(w / 2), y + int(h / 2)

                # Save the points
                if get_dist:  # Gets distance from center
                    center_haystack = int(w_src / 2), int(h_src / 2)
                    dist = u.get_distance(center_haystack, center_needle)
                    points.append((dist, center_needle))

                else:
                    points.append(center_needle)

        return points

    if passed and passed_only:
        return passed

    return False



import ctypes
import time
from random import randint
from time import sleep

LONG = ctypes.c_long
DWORD = ctypes.c_ulong
ULONG_PTR = ctypes.POINTER(DWORD)
WORD = ctypes.c_ushort

INPUT_MOUSE = 0
INPUT_KEYBOARD = 1
INPUT_HARDWARE = 2

KEYEVENTF_EXTENDEDKEY = 0x0001
KEYEVENTF_KEYUP = 0x0002
KEYEVENTF_SCANCODE = 0x0008
KEYEVENTF_UNICODE = 0x0004

VK_RETURN = 0x0D  # ENTER key

UNICODE_TO_VK = {
    '\r': VK_RETURN,
    '\n': VK_RETURN,
}


class MOUSE_INPUT(ctypes.Structure):
    _fields_ = (('dx', LONG),
                ('dy', LONG),
                ('mouseData', DWORD),
                ('dwFlags', DWORD),
                ('time', DWORD),
                ('dwExtraInfo', ULONG_PTR))


class KEYBOARD_INPUT(ctypes.Structure):
    _fields_ = (('wVk', WORD),
                ('wScan', WORD),
                ('dwFlags', DWORD),
                ('time', DWORD),
                ('dwExtraInfo', ULONG_PTR))


class HARDWARE_INPUT(ctypes.Structure):
    _fields_ = (('uMsg', DWORD),
                ('wParamL', WORD),
                ('wParamH', WORD))


class _INPUT_union(ctypes.Union):
    _fields_ = (('mi', MOUSE_INPUT),
                ('ki', KEYBOARD_INPUT),
                ('hi', HARDWARE_INPUT))


class INPUT(ctypes.Structure):
    _fields_ = (('type', DWORD),
                ('union', _INPUT_union))


VK_LBUTTON = 0x01  # Left mouse button
VK_RBUTTON = 0x02  # Right mouse button
VK_CANCEL = 0x03  # Control-break processing
VK_MBUTTON = 0x04  # Middle mouse button (three-button mouse)
VK_XBUTTON1 = 0x05  # X1 mouse button
VK_XBUTTON2 = 0x06  # X2 mouse button
VK_BACK = 0x08  # BACKSPACE key
VK_TAB = 0x09  # TAB key
VK_CLEAR = 0x0C  # CLEAR key
VK_RETURN = 0x0D  # ENTER key
VK_SHIFT = 0x10  # SHIFT key
VK_CONTROL = 0x11  # CTRL key
VK_MENU = 0x12  # ALT key
VK_PAUSE = 0x13  # PAUSE key
VK_CAPITAL = 0x14  # CAPS LOCK key
VK_KANA = 0x15  # IME Kana mode
VK_HANGUL = 0x15  # IME Hangul mode
VK_JUNJA = 0x17  # IME Junja mode
VK_FINAL = 0x18  # IME final mode
VK_HANJA = 0x19  # IME Hanja mode
VK_KANJI = 0x19  # IME Kanji mode
VK_ESCAPE = 0x1B  # ESC key
VK_CONVERT = 0x1C  # IME convert
VK_NONCONVERT = 0x1D  # IME nonconvert
VK_ACCEPT = 0x1E  # IME accept
VK_MODECHANGE = 0x1F  # IME mode change request
VK_SPACE = 0x20  # SPACEBAR
VK_PRIOR = 0x21  # PAGE UP key
VK_NEXT = 0x22  # PAGE DOWN key
VK_END = 0x23  # END key
VK_HOME = 0x24  # HOME key
VK_LEFT = 0x25  # LEFT ARROW key
VK_UP = 0x26  # UP ARROW key
VK_RIGHT = 0x27  # RIGHT ARROW key
VK_DOWN = 0x28  # DOWN ARROW key
VK_SELECT = 0x29  # SELECT key
VK_PRINT = 0x2A  # PRINT key
VK_EXECUTE = 0x2B  # EXECUTE key
VK_SNAPSHOT = 0x2C  # PRINT SCREEN key
VK_INSERT = 0x2D  # INS key
VK_DELETE = 0x2E  # DEL key
VK_HELP = 0x2F  # HELP key
VK_LWIN = 0x5B  # Left Windows key (Natural keyboard)
VK_RWIN = 0x5C  # Right Windows key (Natural keyboard)
VK_APPS = 0x5D  # Applications key (Natural keyboard)
VK_SLEEP = 0x5F  # Computer Sleep key
VK_NUMPAD0 = 0x60  # Numeric keypad 0 key
VK_NUMPAD1 = 0x61  # Numeric keypad 1 key
VK_NUMPAD2 = 0x62  # Numeric keypad 2 key
VK_NUMPAD3 = 0x63  # Numeric keypad 3 key
VK_NUMPAD4 = 0x64  # Numeric keypad 4 key
VK_NUMPAD5 = 0x65  # Numeric keypad 5 key
VK_NUMPAD6 = 0x66  # Numeric keypad 6 key
VK_NUMPAD7 = 0x67  # Numeric keypad 7 key
VK_NUMPAD8 = 0x68  # Numeric keypad 8 key
VK_NUMPAD9 = 0x69  # Numeric keypad 9 key
VK_MULTIPLY = 0x6A  # Multiply key
VK_ADD = 0x6B  # Add key
VK_SEPARATOR = 0x6C  # Separator key
VK_SUBTRACT = 0x6D  # Subtract key
VK_DECIMAL = 0x6E  # Decimal key
VK_DIVIDE = 0x6F  # Divide key
VK_F1 = 0x70  # F1 key
VK_F2 = 0x71  # F2 key
VK_F3 = 0x72  # F3 key
VK_F4 = 0x73  # F4 key
VK_F5 = 0x74  # F5 key
VK_F6 = 0x75  # F6 key
VK_F7 = 0x76  # F7 key
VK_F8 = 0x77  # F8 key
VK_F9 = 0x78  # F9 key
VK_F10 = 0x79  # F10 key
VK_F11 = 0x7A  # F11 key
VK_F12 = 0x7B  # F12 key
VK_F13 = 0x7C  # F13 key
VK_F14 = 0x7D  # F14 key
VK_F15 = 0x7E  # F15 key
VK_F16 = 0x7F  # F16 key
VK_F17 = 0x80  # F17 key
VK_F18 = 0x81  # F18 key
VK_F19 = 0x82  # F19 key
VK_F20 = 0x83  # F20 key
VK_F21 = 0x84  # F21 key
VK_F22 = 0x85  # F22 key
VK_F23 = 0x86  # F23 key
VK_F24 = 0x87  # F24 key
VK_NUMLOCK = 0x90  # NUM LOCK key
VK_SCROLL = 0x91  # SCROLL LOCK key
VK_LSHIFT = 0xA0  # Left SHIFT key
VK_RSHIFT = 0xA1  # Right SHIFT key
VK_LCONTROL = 0xA2  # Left CONTROL key
VK_RCONTROL = 0xA3  # Right CONTROL key
VK_LMENU = 0xA4  # Left MENU key
VK_RMENU = 0xA5  # Right MENU key
VK_BROWSER_BACK = 0xA6  # Browser Back key
VK_BROWSER_FORWARD = 0xA7  # Browser Forward key
VK_BROWSER_REFRESH = 0xA8  # Browser Refresh key
VK_BROWSER_STOP = 0xA9  # Browser Stop key
VK_BROWSER_SEARCH = 0xAA  # Browser Search key
VK_BROWSER_FAVORITES = 0xAB  # Browser Favorites key
VK_BROWSER_HOME = 0xAC  # Browser Start and Home key
VK_VOLUME_MUTE = 0xAD  # Volume Mute key
VK_VOLUME_DOWN = 0xAE  # Volume Down key
VK_VOLUME_UP = 0xAF  # Volume Up key
VK_MEDIA_NEXT_TRACK = 0xB0  # Next Track key
VK_MEDIA_PREV_TRACK = 0xB1  # Previous Track key
VK_MEDIA_STOP = 0xB2  # Stop Media key
VK_MEDIA_PLAY_PAUSE = 0xB3  # Play/Pause Media key
VK_LAUNCH_MAIL = 0xB4  # Start Mail key
VK_LAUNCH_MEDIA_SELECT = 0xB5  # Select Media key
VK_LAUNCH_APP1 = 0xB6  # Start Application 1 key
VK_LAUNCH_APP2 = 0xB7  # Start Application 2 key
VK_OEM_1 = 0xBA  # Used for miscellaneous characters; it can vary by keyboard.
# For the US standard keyboard, the ';:' key
VK_OEM_PLUS = 0xBB  # For any country/region, the '+' key
VK_OEM_COMMA = 0xBC  # For any country/region, the ',' key
VK_OEM_MINUS = 0xBD  # For any country/region, the '-' key
VK_OEM_PERIOD = 0xBE  # For any country/region, the '.' key
VK_OEM_2 = 0xBF  # Used for miscellaneous characters; it can vary by keyboard.
# For the US standard keyboard, the '/?' key
VK_OEM_3 = 0xC0  # Used for miscellaneous characters; it can vary by keyboard.
# For the US standard keyboard, the '`~' key
VK_OEM_4 = 0xDB  # Used for miscellaneous characters; it can vary by keyboard.
# For the US standard keyboard, the '[{' key
VK_OEM_5 = 0xDC  # Used for miscellaneous characters; it can vary by keyboard.
# For the US standard keyboard, the '\|' key
VK_OEM_6 = 0xDD  # Used for miscellaneous characters; it can vary by keyboard.
# For the US standard keyboard, the ']}' key
VK_OEM_7 = 0xDE  # Used for miscellaneous characters; it can vary by keyboard.
# For the US standard keyboard, the 'single-quote/double-quote' key
VK_OEM_8 = 0xDF  # Used for miscellaneous characters; it can vary by keyboard.
VK_OEM_102 = 0xE2  # Either the angle bracket key or the backslash key on the RT 102-key keyboard
VK_PROCESSKEY = 0xE5  # IME PROCESS key
VK_PACKET = 0xE7  # Used to pass Unicode characters as if they were keystrokes. The VK_PACKET key is the low word of
# a 32-bit Virtual Key value used for non-keyboard input methods. For more information, see Remark in KEYBDINPUT,
# SendInput, WM_KEYDOWN, and WM_KEYUP
VK_ATTN = 0xF6  # Attn key
VK_CRSEL = 0xF7  # CrSel key
VK_EXSEL = 0xF8  # ExSel key
VK_EREOF = 0xF9  # Erase EOF key
VK_PLAY = 0xFA  # Play key
VK_ZOOM = 0xFB  # Zoom key
VK_PA1 = 0xFD  # PA1 key
VK_OEM_CLEAR = 0xFE  # Clear key

KEY_0 = 0x30
KEY_1 = 0x31
KEY_2 = 0x32
KEY_3 = 0x33
KEY_4 = 0x34
KEY_5 = 0x35
KEY_6 = 0x36
KEY_7 = 0x37
KEY_8 = 0x38
KEY_9 = 0x39
KEY_A = 0x41
KEY_B = 0x42
KEY_C = 0x43
KEY_D = 0x44
KEY_E = 0x45
KEY_F = 0x46
KEY_G = 0x47
KEY_H = 0x48
KEY_I = 0x49
KEY_J = 0x4A
KEY_K = 0x4B
KEY_L = 0x4C
KEY_M = 0x4D
KEY_N = 0x4E
KEY_O = 0x4F
KEY_P = 0x50
KEY_Q = 0x51
KEY_R = 0x52
KEY_S = 0x53
KEY_T = 0x54
KEY_U = 0x55
KEY_V = 0x56
KEY_W = 0x57
KEY_X = 0x58
KEY_Y = 0x59
KEY_Z = 0x5A


def send_input(*inputs):
    n_inputs = len(inputs)
    lp_input = INPUT * n_inputs
    p_inputs = lp_input(*inputs)
    cb_size = ctypes.c_int(ctypes.sizeof(INPUT))
    return ctypes.windll.user32.SendInput(n_inputs, p_inputs, cb_size)


def input_structure(structure):
    if isinstance(structure, MOUSE_INPUT):
        return INPUT(INPUT_MOUSE, _INPUT_union(mi=structure))
    if isinstance(structure, KEYBOARD_INPUT):
        return INPUT(INPUT_KEYBOARD, _INPUT_union(ki=structure))
    if isinstance(structure, HARDWARE_INPUT):
        return INPUT(INPUT_HARDWARE, _INPUT_union(hi=structure))
    raise TypeError('Cannot create INPUT structure!')


def keyboard_input_unicode(code, flags=0):
    flags = KEYEVENTF_UNICODE | flags
    return KEYBOARD_INPUT(0, code, flags, 0, None)


def keyboard_input_vk(code, flags=0):
    return KEYBOARD_INPUT(code, code, flags, 0, None)


def keyboard_event_unicode(code, _):
    return input_structure(keyboard_input_unicode(code))


def keyboard_event_vk(code, flags=0):
    return input_structure(keyboard_input_vk(code, flags))


def is_pressed(code):
    return ctypes.windll.user32.GetKeyState(code) & 0x8000


def press(code):
    down(code)
    time.sleep(0.1)
    up(code)


def down(code):
    send_input(keyboard_event_vk(code, flags=0))


def up(code):
    send_input(keyboard_event_vk(code, KEYEVENTF_KEYUP))


def delay(seconds: float = 0):
    if seconds == 0:
        sleep((randint(99, 121) * 0.001))
    else:
        sleep(seconds)


def two_keys_combo(key1, key2):
    send_input(keyboard_event_vk(key1), keyboard_event_vk(key2))
    time.sleep(0.1)
    send_input(keyboard_event_vk(key2, KEYEVENTF_KEYUP),
               keyboard_event_vk(key1, KEYEVENTF_KEYUP))


def press_(character):
    if character in UNICODE_TO_VK:
        return press(UNICODE_TO_VK[character])
    code = ord(character)
    send_input(keyboard_event_unicode(code, 0))
    send_input(keyboard_event_unicode(code, KEYEVENTF_KEYUP))


def type_stream(string):
    for char in string:
        press_(char)
        time.sleep(0.05)


def main():
    time.sleep(3)
    # type_stream(u'O\nשש2E6UXoשש2E^uXh#:SHn&HQ'
    send_input(keyboard_event_vk(VK_MENU), keyboard_event_vk(VK_END))
    sleep(0.1)
    send_input(keyboard_event_vk(VK_END, KEYEVENTF_KEYUP),
               keyboard_event_vk(VK_MENU, KEYEVENTF_KEYUP))


if __name__ == '__main__':
    main()



import cv2

import ninja_tools.utils as u
from ninja_tools.processing import Loop

cropping = False
original_image = None
show_cropped_window = False
filename = False
x_start, y_start, x_end, y_end = 0, 0, 0, 0


def mouse_crop(event, x, y, flags, param):
    # grab references to the global variables
    global x_start, y_start, x_end, y_end, cropping, original_image, show_cropped_window, filename

    # if the left mouse button was DOWN, start RECORDING
    # (x, y) coordinates and indicate that cropping is being
    if event == cv2.EVENT_LBUTTONDOWN:
        x_start, y_start, x_end, y_end = x, y, x, y
        cropping = True

    # Mouse is Moving
    elif event == cv2.EVENT_MOUSEMOVE:
        if cropping:
            x_end, y_end = x, y

    # if the left mouse button was released
    elif event == cv2.EVENT_LBUTTONUP:
        # record the ending (x, y) coordinates
        x_end, y_end = x, y
        cropping = False  # cropping is finished
        ref_point = [(x_start, y_start), (x_end, y_end)]

        if len(ref_point) == 2:  # when two points were found
            roi = original_image[ref_point[0][1]:ref_point[1][1], ref_point[0][0]:ref_point[1][0]]

            if show_cropped_window:
                cv2.imshow("Cropped", roi)

            if filename is not False:
                path = f"{filename}_{u.timestamp()}.jpg"
                cv2.imwrite(path, roi)
                print(f"Cropped image saved as: {path}")

            print(f"BBOX(({x_start}, {y_start}, {x_end}, {y_end}))")


cv2.namedWindow("image")
cv2.setMouseCallback("image", mouse_crop)


def crop(img, show_cropped=False, save_as_filename=False):
    global original_image, show_cropped_window, filename
    show_cropped_window = show_cropped
    filename = save_as_filename

    print("Press 'q' to exit/next window.")

    while Loop(25)():
        original_image = img.copy()
        i = img.copy()

        if not cropping:
            cv2.imshow("image", img)

        elif cropping:
            cv2.rectangle(i, (x_start, y_start), (x_end, y_end), (255, 0, 0), 2)
            cv2.imshow("image", i)

        if cv2.waitKey(1) & 0xFF == ord('q'):
            cv2.destroyAllWindows()



from datetime import datetime
from math import sqrt
from time import perf_counter, sleep

from win32gui import GetWindowText, GetForegroundWindow, FindWindow


# Math Functions
def safe_div(x, y):
    return 0 if y == 0 else x / y


def safe_div_round(x, y, decimals=2):
    return round(safe_div(x, y), decimals)


def safe_div_int(x, y):
    return int(0 if y == 0 else x / y)


def get_distance(p0, p1):
    return sqrt((p0[0] - p1[0]) ** 2 + (p0[1] - p1[1]) ** 2)


# Process Functions
def get_handle(window):
    return FindWindow(None, window)


def current_window():
    return GetWindowText(GetForegroundWindow())


def is_current_window(window_name: str):
    return window_name == current_window()


def pause(milliseconds: int):
    sleep(milliseconds * 0.001)


def make_hash(d):
    __ = ''
    for _ in d:
        __ += str(d[_])
    return hash(__)


# Utility Functions
def timestamp():
    (dt, micro) = datetime.utcnow().strftime('%Y%m%d-%H%M%S.%f').split('.')
    dt = "%s.%03d" % (dt, int(micro) * 0.001)
    return dt


def perf():
    return perf_counter()


# I/O
def write_to_file(filename, text, method: str = "a", add_new_line: bool = True):
    with open(filename, method) as file:
        if add_new_line:
            file.write(text + "\n")
        else:
            file.write(text)


def read_file(filename, method="r"):
    return open(filename, method).read()


def read_lines(filename, method="r"):
    return open(filename, method, encoding="utf8").readlines()


# Assorted
def cv2_show(cv2, image, window_name=None, delay=1, stop_key='q'):
    cv2.imshow(window_name, image)
    if cv2.waitKey(delay) & 0xFF == ord(stop_key):
        cv2.destroyAllWindows()



import numpy as np
import win32con
import win32gui
import win32ui

from classes.structures.bbox import BBOX


def capture_window(window: str = None, bbox: BBOX = None, client_rect: bool = False):
    if not window:
        handle = win32gui.GetDesktopWindow()

    else:
        handle = win32gui.FindWindow(None, window)
        if not handle:
            raise Exception(f"Window: {window} not found!")

    if not bbox:
        if client_rect:
            bbox = BBOX(win32gui.GetClientRect(handle))

        else:
            bbox = BBOX(win32gui.GetWindowRect(handle))

        bbox.left, bbox.top = 0, 0

    # get the window image data
    w_dc = win32gui.GetWindowDC(handle)
    dc_obj = win32ui.CreateDCFromHandle(w_dc)
    c_dc = dc_obj.CreateCompatibleDC()
    data_bit_map = win32ui.CreateBitmap()
    data_bit_map.CreateCompatibleBitmap(dc_obj, bbox.width, bbox.height)
    c_dc.SelectObject(data_bit_map)
    c_dc.BitBlt((0, 0), (bbox.width, bbox.height), dc_obj, (bbox.left, bbox.top), win32con.SRCCOPY)

    # convert the raw data into a format opencv can read
    # data_bit_map.SaveBitmapFile(c_dc, 'debug.bmp')
    signed_ints_array = data_bit_map.GetBitmapBits(True)
    img = np.frombuffer(signed_ints_array, dtype='uint8')
    img.shape = (bbox.height, bbox.width, 4)

    # free resources
    dc_obj.DeleteDC()
    c_dc.DeleteDC()
    win32gui.ReleaseDC(handle, w_dc)
    win32gui.DeleteObject(data_bit_map.GetHandle())

    # drop the alpha channel, or cv.matchTemplate() will throw an error like:
    #   error: (-215:Assertion failed) (depth == CV_8U || depth == CV_32F) && type == _templ.type()
    #   && _img.dims() <= 2 in function 'cv::matchTemplate'
    img = img[..., :3]

    # make image C_CONTIGUOUS to avoid errors that look like:
    #   File ... in draw_rectangles
    #   TypeError: an integer is required (got type tuple)
    # see the discussion here:
    # https://github.com/opencv/opencv/issues/14866#issuecomment-580207109
    img = np.ascontiguousarray(img)

    # To get the size use h, w = img.shape[:2], yep height first
    return img



import numpy as np


def most_rgb(img):
    colors, count = np.unique(img.reshape(-1, img.shape[-1]), axis=0, return_counts=True)
    b, g, r = colors[count.argmax()]
    return b, g, r


def transparency(img):
    r, g, b = most_rgb(img)
    img[np.all(img == (r, g, b), axis=-1)] = (255, 255, 255)
    h, w, c = img.shape

    # append Alpha channel -- required for BGRA (Blue, Green, Red, Alpha)
    bgra = np.concatenate([img, np.full((h, w, 1), 255, dtype=np.uint8)], axis=-1)

    # create a mask where white pixels ([255, 255, 255]) are True
    white = np.all(img == [255, 255, 255], axis=-1)

    # change the values of Alpha to 0 for all the white pixels
    bgra[white, -1] = 0

    return bgra


def h_split(img):
    h, w = img.shape[:2]

    h_cut = h // 2
    upper = img[:h_cut, :]
    lower = img[h_cut:, :]

    return upper, lower


def w_split(img):
    h, w = img.shape[:2]

    w_cut = w // 2
    left = img[:w_cut, :]
    right = img[w_cut:, :]

    return left, right
